import { useEffect, useState } from "react"
import { useLtoContext } from "../context/LtoContext"
import { JobNumberToString, TricksterAccount, TricksterCharacter } from "../lib/trickster"
import { keepPreviousData, useQuery } from "@tanstack/react-query"
import { useSessionContext } from "../context/SessionContext"

export const CharacterCard = ({character}:{
  character: TricksterCharacter,
})=>{

  const {activeTable, setActiveTable} = useSessionContext()
  return <>
    <div onClick={()=>{
      setActiveTable(character.path)
    }}
      className={`
      flex flex-col border border-black
      hover:cursor-pointer
      hover:bg-blue-100
      h-full
      p-2 ${character.path === activeTable ? `bg-blue-200 hover:bg-blue-100 border-double border-4` : ""}`}>
      <div className="flex"></div>
      <div className="flex flex-col justify-between h-full">
        <div className="flex flex-col gap-2">
          <div className="flex flex-row justify-center text-md">
            <span>{character.name}</span>
          </div>
          <div className="flex flex-row justify-center">
            {character.base_job  === -8 ?
              <img src={`https://knowledge.lifeto.co/animations/npc/npc041_5.png`}/>
            :
            <img
            src={`https://knowledge.lifeto.co/animations/character/chr00${character.base_job}_13.png`}/>
            }
          </div>
          <div className="flex flex-row gap-1">
            <span>class: </span>
            <span>{JobNumberToString(character.current_job)}</span>
          </div>
        </div>
        <div className="flex flex-row gap-1 text-xs">
          <span>path: </span>
          <span>{character.path}</span>
        </div>

      </div>
    </div>
  </>
}

const PleaseLogin = () => {
  return <><div className="align-center">no characters (not logged in?)</div></>
}

export const CharacterRoulette = ()=>{
  const {API, loggedIn} = useLtoContext()
  const {data:characters} = useQuery({
    queryKey:["characters", API.s.user],
    queryFn: async ()=> {
      return API.GetAccounts().then(x=>{
        if(!x) {
          return undefined
        }
        return x.flatMap(x=>{return x?.characters})
      })
    },
    enabled: loggedIn,
    placeholderData: keepPreviousData,
  })

  if(!characters || characters.length == 0) {
    return <PleaseLogin/>
  }

  return <>
    <div className="flex flex-row overflow-x-scroll gap-4 h-full">
      {characters.map(x=>{
        return <CharacterCard key={x.id} character={x} />
      })}
    </div>
  </>


}
