import { createContext, useContext, useEffect, useState } from "react";
import { LTOApiv0 } from "../lib/lifeto";
import { storage } from "../session_storage";
import { LoginHelper, LogoutHelper } from "../lib/session";

interface LtoContextProps {
  API: LTOApiv0;
  login: (username:string, password:string)=>Promise<void>;
  logout: ()=>void;
  loggedIn: boolean
}

const LtoContext = createContext({} as LtoContextProps);

export const LtoContextProvider = ({ children }: { children: any }) => {
  const [API, setAPI] = useState(new LTOApiv0(storage.GetSession()));
  const login = async (username:string , password:string ) =>{
    console.log("attempting logiun", username)
    return new LoginHelper(username, password).login()
    .catch((e)=>{
      if(e.code == "ERR_BAD_REQUEST") {
        throw "invalid username/password"
      }
      console.warn("throwing error", e)
      throw "unknown error, please report"
    })
    .then((session)=>{
      setAPI(new LTOApiv0(session))
      storage.AddSession(session)
      setLoggedIn(true)
    })  }

    const logout = () => {
      new LogoutHelper().logout().then(()=>{
        storage.RemoveSession()
        localStorage.clear()
        window.location.reload()
      })
    }

    const [loggedIn, setLoggedIn] = useState(false)
    useEffect(()=>{
      if(!API) {
        return
      }
      API?.GetLoggedin().then((x)=>{
        setLoggedIn(x)
      })
    }, [API])

    return <LtoContext.Provider value={{
      API,
      login,
      logout,
      loggedIn
      }}>{children}</LtoContext.Provider>;
};

export const useLtoContext = (): LtoContextProps => {
  const context = useContext<LtoContextProps>(LtoContext);
  if (context === null) {
    throw new Error(
      '"useLtoContext" should be used inside a "LtoContextProvider"',
    );
  }

  return context;
};
