import { createContext, Dispatch, SetStateAction, useContext, useState } from "react";
import { LTOApiv0 } from "../lib/lifeto";

type Setter<T> = React.Dispatch<React.SetStateAction<T | undefined>>;
type MustSetter<T> = React.Dispatch<React.SetStateAction<T>>;
import useLocalStorage from "use-local-storage";
import { OrderTracker } from "../lib/lifeto/order_manager";
import { ColumnSet } from "../lib/table";
import { StoreAccounts, StoreChars, StoreColSet, StoreStr } from "../lib/storage";
import { BasicColumns, ColumnInfo, ColumnName, Columns, DetailsColumns, MoveColumns } from '../lib/columns'

interface SessionContextProps {
  orders: OrderTracker;
  activeTable: string;
  screen: string;
  columns: ColumnSet;
  tags: ColumnSet;
  dirty: number;
  currentSearch: string;

  setActiveTable: Setter<string>
  setScreen: Setter<string>
  setDirty: MustSetter<number>
  setCurrentSearch: MustSetter<string>
}


const _defaultColumn:(ColumnInfo| ColumnName)[] = [
  ...BasicColumns,
  ...MoveColumns,
  ...DetailsColumns,
]


const SessionContext = createContext({} as SessionContextProps);

const dotry =  (x:any, d: any)=>{
  try{
    return x()
  }catch{
    return d

  }
}

export const SessionContextProvider = ({ children }: { children: any }) => {
  const [activeTable, setActiveTable] = useLocalStorage<string>("activeTable","")
  const [screen, setScreen] = useLocalStorage<string>("screen","")
  const [columns ] = useState<ColumnSet>(new ColumnSet(_defaultColumn));
  const [tags ] = useState<ColumnSet>(dotry(()=>StoreColSet.Revive("tags"), new ColumnSet()));

  const [orders ] = useState<OrderTracker>(new OrderTracker());
  const [dirty, setDirty] = useState<number>(0);
  const [currentSearch, setCurrentSearch] = useState<string>("");
  return (
    <SessionContext.Provider value={{
      orders, activeTable, screen, columns, tags, dirty, currentSearch,
      setActiveTable, setScreen, setDirty, setCurrentSearch,
    }}>{children}</SessionContext.Provider>
  );
};

export const useSessionContext = (): SessionContextProps => {
  const context = useContext<SessionContextProps>(SessionContext);
  if (context === null) {
    throw new Error(
      '"useSessionContext" should be used inside a "SessionContextProvider"',
    );
  }

  return context;
};
